/* Copyright (c) 2003, Nokia. All rights reserved */

#ifndef __SYMSERV_APPUI_H__
#define __SYMSERV_APPUI_H__

#include <aknappui.h>

#include "TimerModel.h"
#include "TimeoutNotify.h"
#include <MdaImageConverter.h> 


class CSymServAppView;
class CEmulObject;
class CTimeOutTimer;
class CDynItem;

//class MMdaImageUtilObserver;

/*! 
  @class CSymServAppUi
  
  @discussion An instance of class CSymServAppUi is the UserInterface part of the AVKON
  application framework for the SymServ example application
  */
class CSymServAppUi : public CAknAppUi, MTimeoutNotify, public  MMdaImageUtilObserver
    {
public:
/*!
  @function ConstructL
  
  @discussion Perform the second phase construction of a CSymServAppUi object.
  This needs to be public due to the way the framework constructs the AppUi 
  */
    void ConstructL();

/*!
  @function CSymServAppUi
  
  @discussion Perform the first phase of two phase construction.
  This needs to be public due to the way the framework constructs the AppUi 
  */
    CSymServAppUi();


/*!
  @function ~CSymServAppUi
  
  @discussion Destroy the object
  */
    ~CSymServAppUi();

	// Saves an image to a file
    CMdaImageBitmapToFileUtility* iFileSaver;
	// for saving
    TMdaJfifClipFormat* iFormat;

	CFbsBitmap* iBitmap;
	TRect iRectDisplay;

	TBool iSavingImage;

	RPointerArray<CDynItem> iSessionList;


private:
	int Emulator();

	// Timer control, called everytime the time times out
	void TimedOut();

	void TimerCancel();


	void Wait(TUint mSec);
	TInt LoadMapObject();
	void ParseCmd(const TDesC16 &aDes);
	void EmulatePressKey(const TLex  &aLex);
	TInt SimulateRawEvent(TInt aScanCode);

	void SessionOpen(const TLex  &aLex);
	void SessionDo();

	void SessionResponse(const TDesC8& aDes);

	void EventKeyResponse(const TDesC8& aDes);



void SaveL();
//This function is called when the gif file has been opened or an error has occured in the process
virtual void MiuoOpenComplete(TInt aError);
//This function is called whenever a conversion process has finished
virtual void MiuoConvertComplete(TInt aError);
//This function is called when the bitmap (output) file has been created
virtual void MiuoCreateComplete(TInt aError);



public: // from CAknAppUi
/*!
  @function HandleCommandL
  
  @discussion Handle user menu selections
  @param aCommand The enumerated code for the option selected
  */
    void HandleCommandL(TInt aCommand);

private:
/*! @var iAppView The application view */
    CSymServAppView* iAppView;

	RPointerArray<CEmulObject> iEmulObjectList;


	/** timer object */
	CTimeOutTimer* iTimer;


    };


#endif // __SYMSERV_APPUI_H__

