/* Copyright (c) 2003, Nokia. All rights reserved */

#ifndef __SYMSERV_DOCUMENT_H__
#define __SYMSERV_DOCUMENT_H__


#include <akndoc.h>

class CSymServAppUi;
class CEikApplication;


/*! 
  @class CSymServDocument
  
  @discussion An instance of class CSymServDocument is the Document part of the AVKON
  application framework for the SymServ example application
  */
class CSymServDocument : public CAknDocument
    {
public:

/*!
  @function NewL
  
  @discussion Construct a CSymServDocument for the AVKON application aApp 
  using two phase construction, and return a pointer to the created object
  @param aApp The application creating this document
  @result A pointer to the created instance of CSymServDocument
  */
    static CSymServDocument* NewL(CEikApplication& aApp);

/*!
  @function NewLC
  
  @discussion Construct a CSymServDocument for the AVKON application aApp 
  using two phase construction, and return a pointer to the created object
  @param aApp The application creating this document
  @result A pointer to the created instance of CSymServDocument
  */
    static CSymServDocument* NewLC(CEikApplication& aApp);

/*!
  @function ~CSymServDocument
  
  @discussion Destroy the object
  */
    ~CSymServDocument();

public: // from CAknDocument
/*!
  @function CreateAppUiL 
  
  @discussion Create a CSymServAppUi object and return a pointer to it
  @result A pointer to the created instance of the AppUi created
  */
    CEikAppUi* CreateAppUiL();

private:

/*!
  @function ConstructL
  
  @discussion Perform the second phase construction of a CSymServDocument object
  */
    void ConstructL();

/*!
  @function CSymServDocument
  
  @discussion Perform the first phase of two phase construction 
  @param aApp The application creating this document
  */
    CSymServDocument(CEikApplication& aApp);

    };


#endif // __SYMSERV_DOCUMENT_H__
