/* Copyright (c) 2003, Nokia. All rights reserved */

#include <avkon.hrh>
#include <aknnotewrappers.h> 
#include <apgtask.h>
#include <f32file.h>
#include <e32math.h>

#include <MdaImageConverter.h>  // for MMdaImageUtilObserver

#include "SymServ.pan"
#include "SymServAppUi.h"
#include "SymServAppView.h"
#include "SymServ.hrh"

#include "dynitem.h"
#include "timeouttimer.h"


// The timer timeout resolution is different in the WINS and MARM platforms 
// under wins it is 1/10s, under marm it is 1/64s, also both platforms have
// a different time taken to draw the object
#ifdef __WINS__
//static const TInt KTimerPeriod = 100000*3;
static const TInt KTimerPeriodMSec = 1000;
#else
//static const TInt KTimerPeriod = 1000000*3;
static const TInt KTimerPeriodMSec = 1000;
#endif


void CSymServAppUi::ConstructL()
    {
    BaseConstructL();

    iAppView = CSymServAppView::NewL(ClientRect());
    AddToStackL(iAppView);
	//iAppView->MakeVisible(FALSE);


	// Init a file saver utility
    iFileSaver = CMdaImageBitmapToFileUtility::NewL(*this);
    iFormat = new (ELeave) TMdaJfifClipFormat;

   //Make a snapshot
    iRectDisplay = TRect(0,0,176, 208);
    iBitmap=new(ELeave)CFbsBitmap();
    //Create a bitmap from specified rect
    iBitmap->Create(iRectDisplay.Size(),EColor64K); // Adapt with your display mode


	// Create the timeout timer
	iTimer = CTimeOutTimer::NewL( EPriorityLow, *this);
	// Auto Start
	iTimer->After(KTimerPeriodMSec * 100);

    }

CSymServAppUi::CSymServAppUi() 
	: iTimer(NULL) , iSavingImage(FALSE)                         
    {
	// No implementation required
    }

CSymServAppUi::~CSymServAppUi()
    {

	TimerCancel();
    // Then delete the timer object
    delete iTimer;
    iTimer = NULL;

	iSessionList.ResetAndDestroy();

    if (iAppView)
        {
        iEikonEnv->RemoveFromStack(iAppView);
        delete iAppView;
        iAppView = NULL;
        }
    }

void CSymServAppUi::HandleCommandL(TInt aCommand)
    {
    switch(aCommand)
        {
        case EEikCmdExit:
        case EAknSoftkeyExit:
            Exit();
            break;

        case ESymServCommand1:
            {
            //_LIT(message,"Emulate start");
            //CAknInformationNote* informationNote = new (ELeave) CAknInformationNote;
            //informationNote->ExecuteLD(message);
		

			if( !iTimer->IsActive() )
			{
   				// Set the timeout period
				iTimer->After(KTimerPeriodMSec * 100);
				_LIT(message,"Emulate starting");
				CAknInformationNote* informationNote = new (ELeave) CAknInformationNote;
				informationNote->ExecuteLD(message);
			}
			else
			{
				_LIT(message,"Emulate alredy started");
				CAknInformationNote* informationNote = new (ELeave) CAknInformationNote;
				informationNote->ExecuteLD(message);
			}

            }
            break;

        default:
            Panic(ESymServUi);
            break;
        }
    }








int CSymServAppUi::Emulator()
{


	TUint aKeyCode = -1;

	TFileName filename;
#ifdef __WINS__
	filename = _L("Z:\\cmd.txt ");
#else
	filename = _L("c:\\cmd.txt");
#endif

	// Open file containing encrypted data
    RFs fileSession;
    User::LeaveIfError(fileSession.Connect());
    CleanupClosePushL(fileSession);
    RFile file;
    TInt ret = file.Open(fileSession, filename, EFileRead);
	CleanupClosePushL(file);

	if( KErrNone == ret )
	{	

		

		TInt aFileSize;
		file.Size(aFileSize);
		
		HBufC8* HBufFile = HBufC8::NewMaxLC( aFileSize );
		TPtr8 buffDes = HBufFile->Des();
		buffDes.FillZ();
		file.Read( 0, buffDes );
		file.Close();

#ifdef __WINS__
#else
		// Delete file
		User::LeaveIfError(fileSession.Delete(filename));
#endif		
		// Correct UNICODE
		TBuf<256> bufCommand; bufCommand.Zero();
		bufCommand.Copy( (const unsigned short *)buffDes.Ptr(), aFileSize / 2 ); // 2 symb on char!!
		
		ParseCmd( bufCommand );
		
		CleanupStack::PopAndDestroy(HBufFile);
		
	}
	// Clean up
	CleanupStack::PopAndDestroy(); // Close file 
	CleanupStack::PopAndDestroy(); // Close fileSession	
	
	return aKeyCode;
	
}




// Timer control, called everytime the time times out
void CSymServAppUi::TimedOut()
    {
		Emulator();
		if( !iSavingImage )
			SaveL();

		SessionDo();

		iTimer->After(KTimerPeriodMSec * 200); 
    }


void CSymServAppUi::TimerCancel()
    {
	if( iTimer && iTimer->IsActive() )
			iTimer->Cancel();		
	}

//
//
//
void CSymServAppUi::ParseCmd(const TDesC16 &aDes)
{
	
	TLex aLex( aDes );
	
	TPtrC lblAlias = aLex.NextToken();
	// aLex.Inc();
	
	TBuf<256> aAlias; aAlias.Zero();
	aAlias.Copy(lblAlias);
	
	
	TBuf<128> fnc;
	fnc.Copy(_L("simulate"));

	if( KErrNotFound != aAlias.FindF(fnc) )
	{

		EmulatePressKey(aLex);
		
	}
	else
	{
		fnc.Copy(_L("ftp_connect"));
		if( KErrNotFound != aAlias.FindF(fnc) )
		{
			SessionOpen(aLex);
		}
		else
		{
#ifndef __WINS__
			CAknInformationNote* informationNote = new (ELeave) CAknInformationNote;
			informationNote->ExecuteLD(_L("Alias Unknown"));
#endif
		}
		
	}
}



//
//
//
void CSymServAppUi::SessionOpen(const TLex  &aLex)
{
	TInt aCountParam = 0;
	TInt aTimeOut = 0;
	TLex aLexParam = aLex;

	do
	{
		TPtrC lblParam = aLexParam.NextToken();
		if( lblParam.Length() )
		{
			// aLexParam.Inc();
			//CEikonEnv::Static()->InfoWinL(_L("SessionOpenParam:"), lblParam);
			
			aLexParam.Assign( lblParam );
			aLexParam.Val( aTimeOut );

			aCountParam++;
		}
		else
			break;
		
	}while(1);

	if( 1 == aCountParam )
	{
		CDynItem* item = CDynItem::NewL();
		CleanupStack::PushL(item);

		item->SetTimeOut(aTimeOut);

		User::LeaveIfError(iSessionList.Append(item));
		CleanupStack::Pop(item);

		TBuf8 <128> session(_L8("Session is started Id:"));
		session.AppendNum( item->GetId(), EDecimal);
		session.Append(_L8(" Time out set (McSec):"));
		session.AppendNum( aTimeOut, EDecimal);

		SessionResponse(session);
	}
	else
	{
		SessionResponse(_L8("Error: Many parameters!"));
	}


}


//
//
//
void CSymServAppUi::SessionDo()
{

	for( TInt i = 0; i < iSessionList.Count(); ++i )
	{
		CDynItem* item = iSessionList[i];
		if( item->IsTimeOut() )
		{
			TBuf8 <128> session(_L8("Session is closed, Id:"));
			session.AppendNum( item->GetId(), EDecimal);

			SessionResponse(session);

			iSessionList.Remove(i);
			delete item;
			break; // Only one
		}
	}
}



//
//
//
void CSymServAppUi::SessionResponse(const TDesC8& aDes)
{

			TFileName filename;
#ifdef __WINS__
			filename = _L("Z:\\session.txt");
#else
			filename = _L("c:\\session.txt");
#endif
			
			// Create file
			RFs fileSession;
			User::LeaveIfError(fileSession.Connect());
			CleanupClosePushL(fileSession);
			RFile file;
			TInt ret = file.Create(fileSession, filename, EFileWrite);
			if( KErrNone == ret )
			{
				file.Write( aDes );
			}

			file.Close();

			CleanupClosePushL(file);
			// Clean up
			CleanupStack::PopAndDestroy(); // Close file 
			CleanupStack::PopAndDestroy(); // Close fileSession

}


//
//
//
void CSymServAppUi::EventKeyResponse(const TDesC8& aDes)
{

			TFileName filename;
#ifdef __WINS__
			filename = _L("Z:\\eventkey.txt");
#else
			filename = _L("c:\\eventkey.txt");
#endif
			
			// Create file
			RFs fileSession;
			User::LeaveIfError(fileSession.Connect());
			CleanupClosePushL(fileSession);
			RFile file;
			TInt ret = file.Create(fileSession, filename, EFileWrite);
			if( KErrNone == ret )
			{
				file.Write( aDes );
			}

			file.Close();

			CleanupClosePushL(file);
			// Clean up
			CleanupStack::PopAndDestroy(); // Close file 
			CleanupStack::PopAndDestroy(); // Close fileSession

}



//
//
//
void CSymServAppUi::EmulatePressKey(const TLex  &aLex)
{
	TLex aLexKeyCode = aLex;
	TUint aKeyCode;
	TPtrC lblCmd = aLexKeyCode.NextToken();
	aLexKeyCode.Assign( lblCmd );
	aLexKeyCode.Val( aKeyCode );
	// ...
	if( !SimulateRawEvent(aKeyCode) )
	{
		EventKeyResponse(_L8("Ok"));
	}
	else
	{
		EventKeyResponse(_L8("KErrNotFound"));
	}
}

//
//
//
TInt CSymServAppUi::SimulateRawEvent(TInt aScanCode)
{
	RWsSession aWsSession = CCoeEnv::Static()->WsSession();
	// Get the focus window group id
	TInt id = aWsSession.GetFocusWindowGroup();
	
	// EStdKeyApplication0 
	
	if(id != KErrNotFound) 
	{
			TRawEvent aRawEvent;
			aRawEvent.Set(TRawEvent::EKeyDown,aScanCode);
			aWsSession.SimulateRawEvent(aRawEvent);
			aRawEvent.Set(TRawEvent::EKeyUp,aScanCode);
			aWsSession.SimulateRawEvent(aRawEvent);
			aWsSession.Flush();
	}
	else
	{
		CAknInformationNote* informationNote = new (ELeave) CAknInformationNote;
		informationNote->ExecuteLD(_L("RWsSession KErrNotFound"));
		return 1;
	}
	return 0;
	
	
}


///////////////////////////////////////////////////////////////////////


void CSymServAppUi::Wait(TUint mSec)
{
	RTimer timer;
	timer.CreateLocal();
	TRequestStatus waitStatus;
	timer.After(waitStatus,1000 * mSec);
	User::WaitForRequest(waitStatus);
	timer.Close();
}

// Quality factor for JPG saving
const TInt KJpgSavingQualityFactor = 85;           


void CSymServAppUi::SaveL()
{
	
	TFileName filename;
	
#ifdef __WINS__
	filename = _L("Z:\\capture.jpg");
#else
	filename = _L("c:\\capture.jpg");
#endif
	
	RFs fileSession;
    User::LeaveIfError(fileSession.Connect());
    CleanupClosePushL(fileSession);
    RFile file;
    TInt ret = file.Open(fileSession, filename, EFileRead);
	CleanupClosePushL(file);



	// Clean up
	CleanupStack::PopAndDestroy(); // Close file 
	CleanupStack::PopAndDestroy(); // Close fileSession
	


	if( KErrNotFound == ret )	// KErrNotFound
	{
		// Create Format and Codec
		// quality factor from 0 to 100 (55 used here)
		iFormat->iSettings.iQualityFactor = KJpgSavingQualityFactor; 
		iFormat->iSettings.iSampleScheme = 
			TMdaJpgSettings::TColorSampling(TMdaJpgSettings::EColor420);
		
		TMdaPackage* codec = NULL;
		CleanupStack::PushL(codec);
		
		
		CCoeEnv::Static()->ScreenDevice()->CopyScreenToBitmap(iBitmap,iRectDisplay);
		//iBitmap->DataAddress();
		
		
		
		iFileSaver->CreateL( filename,iFormat, codec, NULL );

		// set flag, can't call this when saving
        iSavingImage = ETrue; 

		
		CleanupStack::PopAndDestroy(1);
	}
	

	
	
	
}



//This function is called when the gif file has been opened or an error has occured in the process
void CSymServAppUi::MiuoOpenComplete(TInt aError)
{
}

//This function is called whenever a conversion process has finished
void CSymServAppUi::MiuoConvertComplete(TInt aError)
{
		// set flag, can save new pictures 
    iSavingImage = EFalse;
}

//This function is called when the bitmap (output) file has been created
void CSymServAppUi::MiuoCreateComplete(TInt aError)
{
    if (( aError == KErrNone) && iFileSaver)
        {
        TRAP( aError, iFileSaver->ConvertL(*iBitmap	/*, TRect(0,0,0,0) , 0*/ ));
        }
    else 
        {
        User::Leave(aError);
        }
}
